"""BI CLI."""

from typing import Annotated

# import json
# from typing import Annotated
import typer
from rich.table import Table

# from slices_bi_client.exceptions import ApiException, BadRequestException, ForbiddenException
from slices_bi_client.models.bare_metal_flavor import BareMetalFlavor
from slices_bi_client.models.vm_flavor import VMFlavor

# from slices_bi_client.models.bare_metal_flavor import BareMetalFlavor
# from slices_bi_client.models.vm_flavor import VMFlavor
# from slices_bi_client.models.bare_metal_flavor_create import BareMetalFlavorCreate
# from slices_bi_client.models.vm_flavor_create import VMFlavorCreate
from . import console  # , err_console
from .bi_context import BIContext

flavors_app = typer.Typer(no_args_is_help=True)


# @flavors_app.command(name="create", rich_help_panel="Admin Commands")
# def create_flavor(
#     ctx: BIContext,
#     friendly_name: Annotated[str, typer.Option(prompt=True, show_default=False, help="Short user readable name/ID")],
#     description: Annotated[str, typer.Option(prompt=True, show_default=False, help="Description of this flavor")],
#     ram: Annotated[int, typer.Option(prompt=True, show_default=False, help="The amount of RAM a flavor has, in MiB")],
#     vcpus: Annotated[
#         int,
#         typer.Option(
#             prompt=True, show_default=False, help="The number of virtual CPUs that will be allocated to the server"
#         ),
#     ],
#     disk: Annotated[
#         int,
#         typer.Option(prompt=True, show_default=False, help="The size of the root disk that will be created in MB."),
#     ],
# ):
#     """Create a flavor.

#     Only available to admin users.
#     """

#     dic = FlavorCreate(
#         friendly_name=friendly_name,
#         description=description,
#         ram_mib=ram,
#         vcpus=vcpus,
#         disk_mb=disk,
#     )

#     try:
#         flavor = ctx.obj.api.get_bi_flavors_api(ctx.obj.site_id, ctx.obj.require_project_id()).create_flavor(dic)
#         console.print(f"[bold green]:computer: Created {friendly_name} as {flavor.id} :white_check_mark:")
#     except BadRequestException as exc:
#         err_console.print("[bold red]Could not create flavor[/] :x:")
#         err_console.print("[red]" + json.loads(exc.body)["detail"])
#         raise typer.Exit(code=1) from None
#     except ForbiddenException:
#         err_console.print("[bold red]You do not have permission to create a flavor.[/] :x:")
#         raise typer.Exit(code=1) from None
#     except ApiException as exc:
#         err_console.print("[bold red]Could not create flavor[/] :x:")
#         detail = json.loads(exc.body)["detail"]
#         if isinstance(detail, list):
#             for validation_error in detail:
#                 err_console.print("[red]" + validation_error["msg"])
#         raise typer.Exit(code=1) from None


def _float_to_str(v: float) -> str:
    if float(round(v)) == v:
        return str(int(v))
    return str(round(v, 3))


def _print_vm_flavors_table(site, flavors_page, show_id: bool):
    if show_id:  # TODO reduce copy-paste
        table = Table(
            "ID",
            "Name",
            "Desc",
            "(v)CPUs",
            "RAM (GiB)",
            "Root Disk (GB)",
            title=f"Slices BI Flavors on {site}",
        )
        for flavor_ in flavors_page.flavors:
            flavor = flavor_.actual_instance
            if isinstance(flavor, VMFlavor):
                table.add_row(
                    flavor.id,
                    flavor.friendly_name,
                    flavor.description,
                    str(flavor.vcpus),
                    _float_to_str(flavor.ram_mib / 1024.0),
                    _float_to_str(flavor.disk_mb / 1000.0),
                )

        console.print(table)
    else:
        table = Table(
            "Name",
            "Desc",
            "(v)CPUs",
            "RAM (GiB)",
            "Root Disk (GB)",
            title=f"Slices BI Flavors on {site}",
        )
        for flavor_ in flavors_page.flavors:
            flavor = flavor_.actual_instance
            if isinstance(flavor, VMFlavor):
                table.add_row(
                    flavor.friendly_name,
                    flavor.description,
                    str(flavor.vcpus),
                    _float_to_str(flavor.ram_mib / 1024.0),
                    _float_to_str(flavor.disk_mb / 1000.0),
                )

        console.print(table)


def _print_bare_metal_flavors_table(site, flavors_page, show_id: bool):
    if show_id:
        table = Table(
            "ID",
            "Name",
            "Desc",
            "CPUs",
            "RAM (GiB)",
            "Root Disk (GB)",
            "Interfaces",
            "GPU",
            title=f"Slices BI Flavors on {site}",
        )
        for flavor_ in flavors_page.flavors:
            flavor = flavor_.actual_instance
            if isinstance(flavor, BareMetalFlavor):
                table.add_row(
                    flavor.id,
                    flavor.friendly_name,
                    flavor.description,
                    f"{flavor.cpu_sockets}x{flavor.cpu_cores}x{flavor.cpu_threads}",
                    _float_to_str(flavor.ram_mib / 1024.0),
                    _float_to_str(flavor.disk_mb / 1000.0),
                    str(flavor.network_interface_count),
                    flavor.gpu_model if flavor.gpu_model else "-",
                )
        console.print(table)
    else:
        table = Table(
            "Name",
            "Desc",
            "CPUs",
            "RAM (GiB)",
            "Root Disk (GB)",
            "Interfaces",
            "GPU",
            title=f"Slices BI Flavors on {site}",
        )
        for flavor_ in flavors_page.flavors:
            flavor = flavor_.actual_instance
            if isinstance(flavor, BareMetalFlavor):
                table.add_row(
                    flavor.friendly_name,
                    flavor.description,
                    f"{flavor.cpu_sockets}x{flavor.cpu_cores}x{flavor.cpu_threads}",
                    _float_to_str(flavor.ram_mib / 1024.0),
                    _float_to_str(flavor.disk_mb / 1000.0),
                    str(flavor.network_interface_count),
                    flavor.gpu_model if flavor.gpu_model else "-",
                )

        console.print(table)


@flavors_app.command(name="list", rich_help_panel="Commands")
# def list_flavors(site: Annotated[str, typer.Option(envvar="SLICES_BI_SITE")]):
def list_flavors(
    ctx: BIContext, show_id: Annotated[bool, typer.Option(help="Show the internal ID of the flavor.")] = False
):
    """List flavors."""
    flavors_page = ctx.obj.api.get_bi_flavors_api(ctx.obj.site_id, ctx.obj.require_project_id()).get_flavors()

    has_bare_metal = False
    has_vm = False

    for flavor_ in flavors_page.flavors:
        flavor = flavor_.actual_instance
        if isinstance(flavor, BareMetalFlavor):
            has_bare_metal = True
        elif isinstance(flavor, VMFlavor):
            has_vm = True

    site = ctx.obj.site_id

    if has_vm:
        _print_vm_flavors_table(flavors_page=flavors_page, site=site, show_id=show_id)
    if has_bare_metal:
        _print_bare_metal_flavors_table(flavors_page=flavors_page, site=site, show_id=show_id)

    # For reason of difference between MB and MiB for disk/ram, see https://en.wikipedia.org/wiki/Gigabyte
    # M/Mi instead of G/Gi everywhere to be flexible for IoT and other small devices
